from numba import cuda, types

@cuda.jit
def add(vector1, vector2):
    """
    Add vector2 inplace to vector1
    """
    tx = cuda.threadIdx.x
    bx = cuda.blockIdx.x
    idx = tx + bx * cuda.blockDim.x
    
    if idx >= vector1.shape[0]:
        return

    vector1[idx] += vector2[idx]
    

# best tpb: 640
@cuda.jit
def add_vector_arrays(arr1, arr2):
    """
    Add's each vector pair.
    Vector pair: v1 + v2
                v1 = arr1[i]
                v2 = arr2[i]
    """
    tx = cuda.threadIdx.x
    bx = cuda.blockIdx.x
    idx = tx + bx * cuda.blockDim.x
    dim = arr1.shape[1]
    
    if idx >= arr1.shape[0]:
        return

    for i in range(dim):
        arr1[idx][i] += arr2[idx][i]