from pygame_visuals.universum import Universum
import pygame
import numpy as np
from pygame_visuals.game import InteractableGame

class CircleUniversum(Universum):
    max_radius = 40

    def __init__(self, positions, velocities, masses) -> None:
        super().__init__(positions, velocities, masses)
        self._gen_colors()

    def _gen_colors(self):
        self.color_heavy = np.random.randint(100, 255, 3)
        self.color_light = np.random.randint(0, 100, 3)

    def process_event(self, event: pygame.event.Event):
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_BACKSPACE:
                self._gen_colors()

    def _draw(self, surface: pygame.Surface, offset: np.ndarray, zoom: float, size: np.ndarray):
        masses = self.masses.copy_to_host()
        positions = self.positions.copy_to_host()
        
        max_mass = np.max(masses)

        for pos, mass in zip(positions, masses):
            # calculate position depending on zoom, drag offset
            xy = (pos + offset) * zoom + size / 2
            percent = mass / max_mass
            color = self.color_heavy * percent + (1 - percent) * self.color_light
            pygame.draw.circle(surface, color, xy, mass * zoom * self.max_radius)

            


if __name__ == "__main__":
    game = InteractableGame(1000, 1000)
    verse = CircleUniversum.random_square(10_000, 2, [0, 0, 1], [1000, 0, 10])
    game.add_gameobject(verse)
    game.run()