from pygame_visuals.universum import Universum
import pygame
import numpy as np
from pygame_visuals.game import InteractableGame
from utils import random_file, files

class ImageUniversum(Universum):
    min_length = 40
    image_idx = 0

    def __init__(self, positions, velocities, masses) -> None:
        super().__init__(positions, velocities, masses)
        self._load_image()
    
    def _load_image(self):
        paths = files("images")
        self.image_idx += 1
        self.image_idx = self.image_idx % len(paths)
        path = paths[self.image_idx]
        print("Loading... ", path)
        with open(path) as f:
            self.image = pygame.image.load(f)
    
    def process_event(self, event: pygame.event.Event):
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_BACKSPACE:
                self._load_image()

    def _draw(self, surface: pygame.Surface, offset: np.ndarray, zoom: float, size: np.ndarray):
        masses = self.masses.copy_to_host()
        positions = self.positions.copy_to_host()
        
        max_mass = np.max(masses)

        for pos, mass in zip(positions, masses):
            # calculate position depending on zoom, drag offset
            xy = (pos + offset) * zoom + size / 2
            percent = mass / max_mass
            length = percent * mass * self.min_length * zoom
            xy -= length / 2
            image = pygame.transform.scale(self.image, (length, length))
            image.set_alpha(percent * 100)
            surface.blit(image, xy)

            


if __name__ == "__main__":
    game = InteractableGame(1000, 1000)
    verse = ImageUniversum.random_square(1000, 2, [-100, -1, 1], [100, 1, 10])
    game.add_gameobject(verse)
    game.run()