from abc import ABC, abstractclassmethod
import pygame
from threading import Thread
from time import time
from pygame import surface
import numpy as np
from pygame import draw
from pygame import mouse
from pygame.transform import scale
from abc import ABC, abstractclassmethod



pygame.font.init()
sysfont = pygame.font.SysFont(pygame.font.get_default_font(), 30)


class GameObject(ABC):
    def init_game(self, game):
        pass

    def draw(self, surface: pygame.Surface):
        pass

    def process_event(self, event: pygame.event.Event):
        pass


class Game():
    width = 1000
    height = 1000
    running = True
    gameObjects: list[GameObject] = []
    dt = float("inf")
    fps = 0

    def __init__(self, width, height) -> None:
        self.width, self.height = width, height
        self.surface = pygame.display.set_mode((self.width, self.height))
        self.screen_size = np.array([width, height])

    def add_gameobject(self, go: GameObject):
        """
        f: function, f(pygame.Surface) -> None
        """
        go.init_game(self)
        self.gameObjects.append(go)

    def _draw_gameObjects(self):
        for go in self.gameObjects:
            go.draw(self.surface)
    
    def _handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False

            for go in self.gameObjects:
                go.process_event(event)

    def run(self):
        while self.running:
            loop_start = time()

            self._run()

            # calc fps
            self.dt = time() - loop_start
            self.dt = self.dt if self.dt else 1
            self.fps = 1 / self.dt

            # draw fps counter
            text = sysfont.render(f"{int(self.fps)} fps", True, (255, 255, 255))
            self.surface.blit(text, text.get_rect().topleft)

            # update screen
            pygame.display.flip()
            self.surface.fill(0)

    def _run(self):
        self._handle_events()
        self._draw_gameObjects()


class DraggableGameObject(GameObject):
    def draw(self, surface: pygame.Surface, offset: np.ndarray, zoom: float, size: np.ndarray):
        pass


class InteractableGame(Game):
    def __init__(self, width, height) -> None:
        super().__init__(width, height)
        self._drag_start_pos = np.zeros(2)
        self.mouse_drag_strength = 1
        self.drag_offset = np.zeros(2)
        self.offset = np.zeros(2)
        self.zoom = 1
        self.zoom_offset = np.zeros(2)
        self.mouse_scroll_offset = np.zeros(2)

    def _draw_gameObjects(self):
        offset = self.drag_offset + self.offset
        for go in self.gameObjects:
            go.draw(self.surface, offset, self.zoom, self.screen_size)
    
    def _handle_events(self):
        mouse = np.array(pygame.mouse.get_pos())

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False

            if event.type == pygame.MOUSEBUTTONDOWN:
                # zooming
                if event.button == 4:
                    self.zoom *= 1.125

                if event.button == 5:
                    self.zoom /= 1.125

                # dragging
                if event.button == 1:
                    self._drag_start_pos = mouse

            # dragging
            if event.type == pygame.MOUSEBUTTONUP:
                if event.button == 1:
                    self.offset += self.drag_offset
                    self.drag_offset = np.zeros(2)
                    self._drag_start_pos = np.zeros(2)

            # dragging
            if event.type == pygame.MOUSEMOTION:
                if np.all(self._drag_start_pos != 0):
                    self.drag_offset = self.mouse_drag_strength * (mouse - self._drag_start_pos) * 1/self.zoom
            
            # go
            for go in self.gameObjects:
                go.process_event(event)


# test
if __name__ == "__main__":
    class Circle(DraggableGameObject):
        def __init__(self, x, y, r) -> None:
            self.pos = np.array([x, y])
            self.r = r
    
        def draw(self, surface: pygame.Surface, offset: np.ndarray, zoom: float, size: np.ndarray):
            xy = (self.pos + offset) * zoom + size / 2
            pygame.draw.circle(surface, 255, xy, self.r * zoom)
        

    v = InteractableGame(1000, 1000)
    for x in range(-500, 500, 100):
        for y in range(-500, 500, 100):
            v.add_gameobject(Circle(x, y, 20))
    v.run()