from pygame_visuals.game import InteractableGame, DraggableGameObject
from nvidia_kernels.gravity import calculate_accelerations_with_mass
from nvidia_kernels.vector import add_vector_arrays
from nvidia_kernels.utils import build_kernel
from nvidia_kernels.vector import add
import pygame
import numpy as np
from numba import cuda


class Universum(DraggableGameObject):
    def __init__(self, positions, velocities, masses) -> None:
        self.positions = positions
        self.velocities = velocities
        self.masses = masses
        self.gravity_kernel = build_kernel(calculate_accelerations_with_mass, self.positions.shape[0], 320)
        self.add_vector_array_kernel = build_kernel(add_vector_arrays, self.positions.shape[0], 640)
    
    @classmethod
    def from_np(cls, positions, velocities, masses, dtype=np.float64):
        p = cuda.to_device(np.array(positions, dtype))
        v =cuda.to_device(np.array(velocities, dtype))
        m = cuda.to_device(np.array(masses, dtype))
        return cls(p, v, m)
    
    @classmethod
    def random_square(cls, amount, dim, low=None, high=None):
        """
        amount: int
        dim: int, dimension 1, 2, 3, ...
        low:  list, lowest value for   [pos, vel, mass]
        high: list, highest value for [pos, vel, mass]
        """
        low = low   if  low else [-100, 0, 1]
        high = high if high else [ 100, 0, 1]
        p = np.random.random([amount, dim]) * high[0]
        v = np.random.random([amount, dim]) * high[1]
        m = np.random.random(amount) * high[2]
        p[p < low[0]] = low[0]
        v[v < low[1]] = low[1]
        m[m < low[2]] = low[2]
        return cls.from_np(p, v, m, dtype=np.float64)

    @classmethod
    def random_circle(cls, amount, radius, dim):
        pass

    def _update_positions(self):
        self.gravity_kernel(self.positions, self.masses, self.velocities)
        self.add_vector_array_kernel(self.positions, self.velocities)

    def _draw(self, surface: pygame.Surface, offset: np.ndarray, zoom: float, size: np.ndarray):
        raise NotImplemented

    def draw(self, *args, **kwargs):
        self._update_positions()
        self._draw(*args, **kwargs)


